#data simulation

rSamejimaIRTinf=function(N,nit,alpha,beta,delta,tau,b0,b1){
  t=matrix(,N,nit)
  for (p in 1:N)
    for (i in 1:nit){{
      C=sample(0:2,1,prob=c(plogis(b1[i]-alpha[i]*tau[p])-plogis(b0[i]-alpha[i]*tau[p]),plogis(b0[i]-alpha[i]*tau[p]),1-plogis(b1[i]-alpha[i]*tau[p])))
      if(C==0) t[p,i]=plogis(alpha[i]*tau[p]+beta[i]+rnorm(1,0,sqrt(delta[i])))
      else t[p,i]=C-1
    }}
  return(t)
}


InfoBoundIRT=function(theta=theta,beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2,model){      
  #si2 is the second canonical parameter (omega, phi, or delta, depending on the model)
  if(model=="samejima") out=rep(alpha^2/si2,length(theta))
  return(out)
}

dBoundIRT=function(x,beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2,model){       #missing is coded as -999
  if(model=="samejima"){
    marg=function(theta,x=x,beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2){
      if(x!=0 & x!=1 & x!=-999) out= (plogis(b1-alpha*theta) - plogis(b0-alpha*theta))*1/(x*(1-x))*dnorm(qlogis(x),beta+alpha*theta,sqrt(si2))
      else if(x==0) out=plogis(b0-alpha*theta)
      else if(x==1) out=1-plogis(b1-alpha*theta)
      return(out*dnorm(theta))
    }
  }
  ou=c()
  for(i in 1:length(x)){
    ou[i]=integrate(marg,-Inf,Inf,x=x[i],beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2)$value
  }
  return(ou)
}

#density and information functions

InfoBoundIRTinf=function(theta=theta,beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2,model){     # see appendix A from the paper
  ou=c()                                                                                       #si2 is the second canonical parameter (omega, phi, or delta, depending on the model)
  for(i in 1:length(theta)){
    ou[i]=alpha^2*plogis(-(b0-alpha*theta[i]))^2*plogis((b0-alpha*theta[i]))+                     #info lower bound
      (plogis((b1-alpha*theta[i]))-plogis((b0-alpha*theta[i])))*
      ((-alpha*(plogis((b1-alpha*theta[i]))*plogis(-(b1-alpha*theta[i]))-plogis((b0-alpha*theta[i]))*plogis(-(b0-alpha*theta[i])))/(plogis((b1-alpha*theta[i]))-plogis((b0-alpha*theta[i]))))^2+  #info 0<X<1
         InfoBoundIRT(theta=theta[i],beta=beta,alpha=alpha,b0=b0,b1=b1,si2=si2,model))+       # info traditional model
      alpha^2*plogis((b1-alpha*theta[i]))^2*plogis(-(b1-alpha*theta[i]))                       #info upper bound
  }
  return(ou)
}

#fit functions

fitSamejimaIRTinf=function(t,iter=2000){
  z=t
  z[z!=0 & z!=1 & z!=-999]=qlogis(z[z!=0 & z!=1 & z!=-999])
  N=nrow(t)
  nit=ncol(t)
  fit <- sampling(SamejimaIRTinf, iter=iter, chains=1, data = list(N=N,z=z,nit=nit),init=list(list(
    ln_phi=rnorm(nit,0,.2),
    ln_sigma2=rnorm(nit,0,.2),
    icept=rnorm(nit,1,.5),
    tau=rnorm(N,0,1),
    b0=-abs(rnorm(nit,1,.2)),
    b1=abs(rnorm(nit,1,.2))
  )))
  tau.est=apply(extract(fit,"tau")$tau,2,mean)
  icept.est=apply(extract(fit,"icept")$icept,2,mean)
  phi.est=apply(extract(fit,"phi")$phi,2,mean)
  sigma2.est=apply(extract(fit,"sigma2")$sigma2,2,mean)
  b0.est=apply(extract(fit,"b0")$b0,2,mean)
  b1.est=apply(extract(fit,"b1")$b1,2,mean)
  
  tau.sd=apply(extract(fit,"tau")$tau,2,sd)
  icept.sd=apply(extract(fit,"icept")$icept,2,sd)
  phi.sd=apply(extract(fit,"phi")$phi,2,sd)
  sigma2.sd=apply(extract(fit,"sigma2")$sigma2,2,sd)
  b0.sd=apply(extract(fit,"b0")$b0,2,sd)
  b1.sd=apply(extract(fit,"b1")$b1,2,sd)
  
  output=cbind(icept.est,icept.sd,phi.est,phi.sd,sigma2.est,sigma2.sd,b0.est,b0.sd,b1.est,b1.sd)
  colnames(output)=c("beta_i","sd","alpha_i","sd","delta_i","sd","gamma0_i","sd","gamma1_i","sd")
  rownames(output)=paste("item",1:nit,sep="")
  
  return(list(output=output,theta.est=tau.est,beta.est=icept.est,alpha.est=phi.est,delta.est=sigma2.est,b0.est=b0.est,b1.est=b1.est,
              theta.sd=tau.sd,beta.sd=icept.sd,alpha.sd=phi.sd,delta.sd=sigma2.sd,b0.sd=b0.sd,b1.sd=b1.sd,mcmc=fit))
}

